import { appConfigDir } from '@tauri-apps/api/path'
import { exists, writeTextFile, readTextFile, mkdir } from '@tauri-apps/plugin-fs'
import { type ToastServiceMethods } from 'primevue'
import { type AppConfig } from '../types/data'

export const configDirPath = async () => {
  const configDirPath = await appConfigDir()
  return `${configDirPath}/config.json`
}

/**
 * Gets the app config; if it doesn't exist, it will be made with defaults
 * @param toast the primevue toast instance
 * @returns app config
 */
export const getConfig = async (toast: ToastServiceMethods): Promise<AppConfig> => {
  const configPath = await configDirPath()
  let configExists = false
  try {
    configExists = await exists(configPath)
  } catch (err) {
    toast.add({
      severity: 'error',
      summary: 'The path could not be read',
      detail: err,
    })
  }
  let config = {
    configVersion: 0,
    recentFiles: [],
  }
  let json: AppConfig | null = null

  if (!configExists) {
    const acd = await appConfigDir()
    await mkdir(acd, { recursive: true })
    await writeJson(configPath, config, toast)
  } else {
    json = await readJson<AppConfig>(configPath, toast)
  }

  return json || config
}

/**
 * Saves the app config
 * @param config the app config
 * @param toast the primevue toast instance
 */
export const saveConfig = async (config: AppConfig, toast: ToastServiceMethods) => {
  const configPath = await configDirPath()
  await writeJson(configPath, config, toast)
}

/**
 * Reads json with error handling
 * @param path the full path to read from
 * @param toast the primevue toast instance
 * @returns the parsed json object or null if invalid/unable to read
 */
export const readJson = async<T>(path: string, toast: ToastServiceMethods) => {
  let fileContent: string
  let parsedContent: T
  try {
    fileContent = await readTextFile(path)
  } catch (err) {
    toast.add({
      severity: 'error',
      summary: 'The file could not be read (permission issue?)',
      detail: err,
    })

    return null
  }
  try {
    parsedContent = JSON.parse(fileContent)
  } catch (err) {
    toast.add({
      severity: 'error',
      summary: 'The file could not be parsed (invalid json)',
      detail: err,
    })

    return null
  }

  return parsedContent
}

/**
 * Saves json with error handling
 * @param path the full path to write to
 * @param content the content to write
 * @param toast the primevue toast instance
 */
export const writeJson = async<T>(path: string, content: T, toast: ToastServiceMethods) => {
  let json = ''
  try {
    json = JSON.stringify(content, null, 2)
  } catch (err) {
    toast.add({
      severity: 'error',
      summary: 'The file could not be stringified',
      detail: err,
    })

    return
  }
  try {
    await writeTextFile(path, json)
  } catch (err) {
    toast.add({
      severity: 'error',
      summary: 'The file could not be written (permission issue?)',
      detail: err,
    })
  }
}
